@@ -12,10 +12,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # dataset.py
-# Copyright (C) 2014-2022 Fracpete (pythonwekawrapper at gmail dot com)
+# Copyright (C) 2014-2024 Fracpete (pythonwekawrapper at gmail dot com)
 
 import javabridge
 import logging
+import math
 import numpy as np
 from weka.core.classes import JavaObject, Random
 import weka.core.typeconv as typeconv
@@ -966,7 +967,10 @@ class Instance(JavaObject):
         jni_classname = classname.replace(".", "/")
         if type(values) is list:
             for i in range(len(values)):
-                values[i] = float(values[i])
+                if (values[i] is None) or math.isnan(values[i]):
+                    values[i] = np.nan
+                else:
+                    values[i] = float(values[i])
             values = np.array(values)
         return Instance(
             javabridge.make_instance(
@@ -1743,11 +1747,12 @@ def check_col_names_unique(cols_x, col_y=None):
     return None
 
 
-def create_instances_from_lists(x, y=None, name="data", cols_x=None, col_y=None):
+def create_instances_from_lists(x, y=None, name="data", cols_x=None, col_y=None, nominal_x=None, nominal_y=False):
     """
     Allows the generation of an Instances object from a list of lists for X and a list for Y (optional).
     Data can be numeric, string or bytes. Attributes can be converted to nominal with the
     weka.filters.unsupervised.attribute.NumericToNominal filter.
+    None values are interpreted as missing values.
 
     :param x: the input variables (row wise)
     :type x: list of list
@@ -1759,6 +1764,10 @@ def create_instances_from_lists(x, y=None, name="data", cols_x=None, col_y=None)
     :type cols_x: list
     :param col_y: the column name to use for the output variable (y)
     :type col_y: str
+    :param nominal_x: the list of 0-based column indices to treat as nominal ones, ignored if None
+    :type nominal_x: list
+    :param nominal_y: whether the y column is to be treated as nominal
+    :type nominal_y: bool
     :return: the generated dataset
     :rtype: Instances
     """
@@ -1778,77 +1787,145 @@ def create_instances_from_lists(x, y=None, name="data", cols_x=None, col_y=None)
     if msg is not None:
         raise Exception(msg)
 
+    # nominal x columns?
+    nominal_x_values = None
+    if nominal_x is not None:
+        nominal_x_values = dict()
+        for nominal_col in nominal_x:
+            labels = set()
+            for n in range(len(x)):
+                labels.add(typeconv.to_string(x[n][nominal_col]))
+            nominal_x_values[nominal_col] = sorted(list(labels))
+
     # create header
     atts = []
     type_x = []
     for i in range(len(x[0])):
-        if isinstance(x[0][i], float) or isinstance(x[0][i], int):
-            type_x.append("N")
+        type_x.append(None)
+    for i in range(len(x[0])):
+        for n in range(len(x)):
+            if x[n][i] is None:
+                continue
+            if (nominal_x_values is not None) and (i in nominal_x_values):
+                type_x[i] = "C"
+                break
+            if isinstance(x[n][i], float) or isinstance(x[n][i], int):
+                type_x[i] = "N"
+                break
+            elif isinstance(x[n][i], bytes):
+                type_x[i] = "B"
+                break
+            elif isinstance(x[n][i], str):
+                type_x[i] = "S"
+                break
+            else:
+                raise Exception("Only float, int, bytes and str are supported, #" + str(i) + ": " + str(type(x[i][0])))
+
+    for i in range(len(type_x)):
+        if type_x[i] == "N":
             atts.append(Attribute.create_numeric(cols_x[i]))
-        elif isinstance(x[0][i], bytes):
-            type_x.append("B")
+        elif type_x[i] == "B":
             atts.append(Attribute.create_string(cols_x[i]))
-        elif isinstance(x[0][i], str):
-            type_x.append("S")
+        elif type_x[i] == "S":
             atts.append(Attribute.create_string(cols_x[i]))
+        elif type_x[i] == "C":
+            atts.append(Attribute.create_nominal(cols_x[i], nominal_x_values[i]))
         else:
-            raise Exception("Only float, int, bytes and str are supported, #" + str(i) + ": " + str(type(x[i][0])))
+            print("WARNING: Failed to determine data type for column #%d" % i)
+            atts.append(Attribute.create_numeric(cols_x[i]))
 
     type_y = ""
     if y is not None:
-        if isinstance(y[0], float) or isinstance(y[0], int):
-            type_y = "N"
-            atts.append(Attribute.create_numeric(col_y))
-        elif isinstance(y[0], bytes):
-            type_y = "B"
-            atts.append(Attribute.create_string(col_y))
-        elif isinstance(y[0], str):
-            type_y = "S"
-            atts.append(Attribute.create_string(col_y))
-        else:
-            raise Exception("Only float, int, bytes and str are supported for y: " + str(type(y[0])))
+        # nominal y column?
+        nominal_y_values = None
+        if nominal_y:
+            labels = set()
+            for n in range(len(y)):
+                labels.add(typeconv.to_string(y[n]))
+            nominal_y_values = sorted(list(labels))
+
+        for n in range(len(y)):
+            if y[n] is None:
+                continue
+            if nominal_y:
+                type_y = "C"
+                atts.append(Attribute.create_nominal(col_y, nominal_y_values))
+                break
+            if isinstance(y[n], float) or isinstance(y[n], int):
+                type_y = "N"
+                atts.append(Attribute.create_numeric(col_y))
+                break
+            elif isinstance(y[n], bytes):
+                type_y = "B"
+                atts.append(Attribute.create_string(col_y))
+                break
+            elif isinstance(y[n], str):
+                type_y = "S"
+                atts.append(Attribute.create_string(col_y))
+                break
+            else:
+                raise Exception("Only float, int, bytes and str are supported for y: " + str(type(y[0])))
 
     result = Instances.create_instances(name, atts, len(x))
 
     # add data
+    nan = float("nan")
     for i in range(len(x)):
         values = []
+
         for n in range(len(x[i])):
+            if x[i][n] == nan:
+                values.append(x[i][n])
+                continue
             if type_x[n] == "N":
                 values.append(x[i][n])
             elif type_x[n] == "B":
-                values.append(result.attribute(n).add_string_value(x[i][n].decode("utf-8")))
+                values.append(result.attribute(n).add_string_value(typeconv.to_string(x[i][n])))
+            elif type_x[n] == "C":
+                values.append(result.attribute(n).index_of(typeconv.to_string(x[i][n])))
             else:
                 values.append(result.attribute(n).add_string_value(x[i][n]))
+
         if y is not None:
+            if y[i] == nan:
+                values.append(y[i])
+                continue
             if type_y == "N":
                 values.append(y[i])
             elif type_y == "B":
-                values.append(result.attribute(result.num_attributes - 1).add_string_value(y[i].decode("utf-8")))
+                values.append(result.attribute(result.num_attributes - 1).add_string_value(typeconv.to_string(y[i])))
+            elif type_y == "C":
+                values.append(result.attribute(result.num_attributes - 1).index_of(typeconv.to_string(y[i])))
             else:
                 values.append(result.attribute(result.num_attributes - 1).add_string_value(y[i]))
+
         result.add_instance(Instance.create_instance(values))
 
     return result
 
 
-def create_instances_from_matrices(x, y=None, name="data", cols_x=None, col_y=None):
+def create_instances_from_matrices(x, y=None, name="data", cols_x=None, col_y=None, nominal_x=None, nominal_y=False):
     """
     Allows the generation of an Instances object from a 2-dimensional matrix for X and a
     1-dimensional matrix for Y (optional).
     Data can be numeric, string or bytes. Attributes can be converted to nominal with the
     weka.filters.unsupervised.attribute.NumericToNominal filter.
+    nan values are interpreted as missing values.
 
     :param x: the input variables
-    :type x: ndarray
+    :type x: np.ndarray
     :param y: the output variable (optional)
-    :type y: ndarray
+    :type y: np.ndarray
     :param name: the name of the dataset
     :type name: str
     :param cols_x: the column names to use
     :type cols_x: list
     :param col_y: the column name to use for the output variable (y)
     :type col_y: str
+    :param nominal_x: the list of 0-based column indices to treat as nominal ones, ignored if None
+    :type nominal_x: list
+    :param nominal_y: whether the y column is to be treated as nominal
+    :type nominal_y: bool
     :return: the generated dataset
     :rtype: Instances
     """
@@ -1868,11 +1945,25 @@ def create_instances_from_matrices(x, y=None, name="data", cols_x=None, col_y=No
     if msg is not None:
         raise Exception(msg)
 
+    # nominal x columns?
+    nominal_x_values = None
+    if nominal_x is not None:
+        nominal_x_values = dict()
+        for nominal_col in nominal_x:
+            labels = set()
+            for n in range(len(x)):
+                labels.add(typeconv.to_string(x[n][nominal_col]))
+            nominal_x_values[nominal_col] = sorted(list(labels))
+
     # create header
     atts = []
     type_x = []
     for i in range(len(x[0])):
         try:
+            if (nominal_x_values is not None) and (i in nominal_x_values):
+                type_x.append("C")  # nominal
+                atts.append(Attribute.create_nominal(cols_x[i], nominal_x_values[i]))
+                continue
             len(x.dtype)
             if np.issubdtype(x.dtype[i], np.number):
                 type_x.append("N")  # number
@@ -1888,7 +1979,18 @@ def create_instances_from_matrices(x, y=None, name="data", cols_x=None, col_y=No
             atts.append(Attribute.create_numeric(cols_x[i]))
     type_y = ""
     if y is not None:
-        if np.issubdtype(y.dtype, np.number):
+        # nominal y column?
+        nominal_y_values = None
+        if nominal_y:
+            labels = set()
+            for n in range(len(y)):
+                labels.add(typeconv.to_string(y[n]))
+            nominal_y_values = sorted(list(labels))
+
+        if nominal_y:
+            type_y = "C"
+            atts.append(Attribute.create_nominal(col_y, nominal_y_values))
+        elif np.issubdtype(y.dtype, np.number):
             type_y = "N"  # number
             atts.append(Attribute.create_numeric(col_y))
         elif np.issubdtype(y.dtype, np.str_):
@@ -1903,20 +2005,31 @@ def create_instances_from_matrices(x, y=None, name="data", cols_x=None, col_y=No
     # add data
     for i in range(len(x)):
         values = []
+
         for n in range(len(x[i])):
-            if type_x[n] == "N":
+            if isinstance(x[i][n], float) and np.isnan(x[i][n]):
+                values.append(missing_value())
+            elif type_x[n] == "N":
                 values.append(x[i][n])
             elif type_x[n] == "S":
                 values.append(result.attribute(n).add_string_value(x[i][n]))
+            elif type_x[n] == "C":
+                values.append(result.attribute(n).index_of(typeconv.to_string(x[i][n])))
             else:
-                values.append(result.attribute(n).add_string_value(x[i][n].decode("utf-8")))
+                values.append(result.attribute(n).add_string_value(typeconv.to_string(x[i][n])))
+
         if y is not None:
-            if type_y == "N":
+            if isinstance(y[i], float) and np.isnan(y[i]):
+                values.append(missing_value())
+            elif type_y == "N":
                 values.append(y[i])
             elif type_y == "S":
                 values.append(result.attribute(result.num_attributes - 1).add_string_value(y[i]))
+            elif type_y == "C":
+                values.append(result.attribute(result.num_attributes - 1).index_of(typeconv.to_string(y[i])))
             else:
-                values.append(result.attribute(result.num_attributes - 1).add_string_value(y[i].decode("utf-8")))
+                values.append(result.attribute(result.num_attributes - 1).add_string_value(typeconv.to_string(y[i])))
+
         result.add_instance(Instance.create_instance(values))
 
     return result
